<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:dp="http://www.datapower.com/extensions"
  xmlns:regexp="http://exslt.org/regular-expressions"
  xmlns:str="http://exslt.org/strings"
  xmlns:func="http://exslt.org/functions"
  extension-element-prefixes="dp"
>
  <dp:input-mapping href="store:///pkcs7-convert-input.ffd" type="ffd"/>

  <xsl:output omit-xml-declaration="yes" />


  <xsl:template match="/">
    <!-- base64 encode Non-XML input data -->
    <xsl:variable name="input64"
      select="dp:binary-encode(/object/message/node())"/>

    <!-- base64 decode; results in input in case of valid UTF-8 -->
    <xsl:variable name="input"
      select="dp:decode($input64, 'base-64')"/>

    <!-- determine bounday from mime headers -->
    <xsl:variable name="boundary" 
      select="substring-before(
                substring-after($input,'boundary=&quot;'),
                '&quot;'
              )"/>

    <!-- determine first quoted-prinatble part -->
    <xsl:variable name="input1"
      select="substring-after(
                $input,
                'Content-Transfer-Encoding: quoted-printable'
              )"/>

    <!-- skip over header values separated by empy line -->
    <xsl:variable name="input2"
      select="substring-after($input1,'&#13;&#10;&#13;&#10;')"/>

    <!-- take anything until next boundary -->
    <xsl:variable name="input3"
      select="substring-before($input2,concat('--',$boundary))"/>

    <!-- output data as is and quoted-printable decoded -->
    <xsl:value-of select="$input3"/>
    <xsl:text>----------------------------------------&#10;</xsl:text>
    <xsl:copy-of select="func:qp($input3)"/>

    <!-- 
         Parsing has to skip xml-declaration because of DataPower internal 
         UTF-8 encoding, for details see these postings:
         https://www.ibm.com/developerworks/mydeveloperworks/blogs/HermannSW/tags/encoding
    -->
    <xsl:text>----------------------------------------&#10;</xsl:text>
    <xsl:copy-of select="dp:parse(substring-after(func:qp($input3),'?>'))"/>

  </xsl:template>
  

  <!-- decode quoted-prinatble string -->
  <func:function name="func:qp">
    <xsl:param name="str"/>

    <!-- eliminate soft line breaks -->
    <xsl:variable name="wosoft" 
      select="regexp:replace($str,'=\r{0,1}\n','','')"/>

    <!-- eliminate '=' characters (encoded as =3D) by splitting -->
    <xsl:variable name="tokens" select="str:split($wosoft,'=3D')"/>

    <func:result>
      <!-- decode first token -->
      <xsl:value-of select="func:qp-rec($tokens[1])"/>

      <!-- for all other tokes -->
      <xsl:for-each select="$tokens[position() > 1]">
        <!-- output '=' (for =3D) -->
        <xsl:text>=</xsl:text>

        <!-- decode token -->
        <xsl:value-of select="func:qp-rec(.)"/>
      </xsl:for-each>
    </func:result>
  </func:function>


  <!-- 
       Recursive quoted-printable decoding.
       Precondition: 
       - no soft line breaks ('=' at end of line)
       - no '=3D' contained
  -->
  <func:function name="func:qp-rec">
    <xsl:param name="str"/>

    <xsl:choose>
      <!-- if $str does not contaion a '=' we are done -->
      <xsl:when test="not(contains($str,'='))"> 
        <func:result select="$str"/>
      </xsl:when>

      <xsl:otherwise>
        <!-- determine index of first quoted-printable character -->
        <xsl:variable name="idx" 
          select="substring(substring-after($str,'='), 1, 2)"/>

        <!-- map to corresponding parsed character value -->
        <xsl:variable name="m" 
          select="substring($map, 1+dp:radix-convert($idx,16,10), 1)"/>

        <!-- replace all occurences -->
        <func:result 
          select="func:qp-rec(regexp:replace($str,concat('=',$idx),'g',$m))"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>


  <!-- 
       Map for 8bit characters; Non-UTF8 characters are mapped to '_'.
       This mapping is for ISO-8859-1, please adapt if necessary.
  -->
  <xsl:variable name="apos" select='"&apos;"'/>

  <xsl:variable name="map" 
    select="concat('&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;',
                   '&#x5F;&#x09;&#x0A;&#x5F;&#x5F;&#x0D;&#x5F;&#x5F;',
                   '&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;',
                   '&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;&#x5F;',
                   '&#x20;&#x21;&#x22;&#x23;&#x24;&#x25;&#x26;',$apos,
                   '&#x28;&#x29;&#x2A;&#x2B;&#x2C;&#x2D;&#x2E;&#x2F;',
                   '&#x30;&#x31;&#x32;&#x33;&#x34;&#x35;&#x36;&#x37;',
                   '&#x38;&#x39;&#x3A;&#x3B;&#x3C;&#x3D;&#x3E;&#x3F;',
                   '&#x40;&#x41;&#x42;&#x43;&#x44;&#x45;&#x46;&#x47;',
                   '&#x48;&#x49;&#x4A;&#x4B;&#x4C;&#x4D;&#x4E;&#x4F;',
                   '&#x50;&#x51;&#x52;&#x53;&#x54;&#x55;&#x56;&#x57;',
                   '&#x58;&#x59;&#x5A;&#x5B;&#x5C;&#x5D;&#x5E;&#x5F;',
                   '&#x60;&#x61;&#x62;&#x63;&#x64;&#x65;&#x66;&#x67;',
                   '&#x68;&#x69;&#x6A;&#x6B;&#x6C;&#x6D;&#x6E;&#x6F;',
                   '&#x70;&#x71;&#x72;&#x73;&#x74;&#x75;&#x76;&#x77;',
                   '&#x78;&#x79;&#x7A;&#x7B;&#x7C;&#x7D;&#x7E;&#x7F;',
                   '&#x80;&#x81;&#x82;&#x83;&#x84;&#x85;&#x86;&#x87;',
                   '&#x88;&#x89;&#x8A;&#x8B;&#x8C;&#x8D;&#x8E;&#x8F;',
                   '&#x90;&#x91;&#x92;&#x93;&#x94;&#x95;&#x96;&#x97;',
                   '&#x98;&#x99;&#x9A;&#x9B;&#x9C;&#x9D;&#x9E;&#x9F;',
                   '&#xA0;&#xA1;&#xA2;&#xA3;&#xA4;&#xA5;&#xA6;&#xA7;',
                   '&#xA8;&#xA9;&#xAA;&#xAB;&#xAC;&#xAD;&#xAE;&#xAF;',
                   '&#xB0;&#xB1;&#xB2;&#xB3;&#xB4;&#xB5;&#xB6;&#xB7;',
                   '&#xB8;&#xB9;&#xBA;&#xBB;&#xBC;&#xBD;&#xBE;&#xBF;',
                   '&#xC0;&#xC1;&#xC2;&#xC3;&#xC4;&#xC5;&#xC6;&#xC7;',
                   '&#xC8;&#xC9;&#xCA;&#xCB;&#xCC;&#xCD;&#xCE;&#xCF;',
                   '&#xD0;&#xD1;&#xD2;&#xD3;&#xD4;&#xD5;&#xD6;&#xD7;',
                   '&#xD8;&#xD9;&#xDA;&#xDB;&#xDC;&#xDD;&#xDE;&#xDF;',
                   '&#xE0;&#xE1;&#xE2;&#xE3;&#xE4;&#xE5;&#xE6;&#xE7;',
                   '&#xE8;&#xE9;&#xEA;&#xEB;&#xEC;&#xED;&#xEE;&#xEF;',
                   '&#xF0;&#xF1;&#xF2;&#xF3;&#xF4;&#xF5;&#xF6;&#xF7;',
                   '&#xF8;&#xF9;&#xFA;&#xFB;&#xFC;&#xFD;&#xFE;&#xFF;'
                  )"/>
    
</xsl:stylesheet>
